import axios, { AxiosError, AxiosInstance } from 'axios';
import { API_QUERY_REFERER, API_URL } from '../config';
import { GiteaPublicProperties } from '../model/giteaPublicProperties.model';
import { SshKeyPair } from '../model/sshKeyPair.model';
import { Attempt } from '../model/attempt';
import Controller from './controller';
import ExtensionStore from './extensionStore';
import KeycloakOAuth2DeviceFlowConnection from './keycloakOAuth2DeviceFlowConnection';
import { MissionUser } from './mission/model/missionUser';
import UserMission, { User } from './mission/model/userMission';
import { error } from './utils/log';

export default class ApiService {
  private axiosInstance: AxiosInstance;
  private static _instance: ApiService;

  static get instance(): ApiService {
    if (!ApiService._instance) {
      ApiService._instance = new ApiService();
    }
    return ApiService._instance;
  }

  private constructor() {
    this.axiosInstance = axios.create({
      baseURL: API_URL,
      headers: {
        'Content-Type': 'application/json',
        referer: API_QUERY_REFERER,
      },
    });

    this.initApiInterceptor();
  }

  initApiInterceptor() {
    this.initRequestInterceptor();
    this.initResponseInterceptor();
  }

  private initRequestInterceptor() {
    this.axiosInstance.interceptors.request.use(
      async (config) => {
        const accessToken = await ExtensionStore.instance.getAccessToken();
        if (accessToken && config.headers) {
          config.headers['authorization'] = `BEARER ${accessToken}`;
        }
        return config;
      },
      (error) => {
        return Promise.reject(error);
      },
    );
  }

  private initResponseInterceptor() {
    this.axiosInstance.interceptors.response.use(
      (res) => {
        return res;
      },
      async (err) => {
        return await this.handleResponseError(err);
      },
    );
  }

  private async handleResponseError(err) {
    const originalConfig = err.config;
    if (err.response) {
      // Access Token was expired
      if (err.response.status === 401) {
        if (!originalConfig._retry) {
          originalConfig._retry = true;
          return await this.onRetry(originalConfig);
        } else if (originalConfig._retry) {
          // IF REFRESH TOKEN NOT WORK, REQUEST NEW CONNECTION IN USER BROWSER
          return await this.onInvalidRefreshToken(originalConfig);
        }
      }
      if (err.response.status === 403 && err.response.data) {
        return Promise.reject(err.response.data);
      }
    }
    return Promise.reject(err);
  }

  private async onRetry(originalConfig) {
    try {
      const storedRefreshToken = await ExtensionStore.instance.getRefreshToken();
      const { accessToken, refreshToken } = await KeycloakOAuth2DeviceFlowConnection.instance.getToken({
        refreshToken: storedRefreshToken,
        openLink: Controller.openBrowserWithUrl,
      });

      await ExtensionStore.instance.setAccessToken(accessToken);
      await ExtensionStore.instance.setRefreshToken(refreshToken);

      this.axiosInstance.defaults.headers.common['authorization'] = `BEARER ${accessToken}`;
      return this.axiosInstance(originalConfig);
    } catch (_error) {
      if (_error instanceof AxiosError) {
        if (_error.response && _error.response.data) {
          return Promise.reject(_error.response.data);
        } else {
          return await this.onInvalidRefreshToken(originalConfig);
        }
      } else {
        error(`Error not handled: ${_error}`);
        throw _error;
      }
    }
  }

  private async onInvalidRefreshToken(originalConfig) {
    try {
      if (await Controller.instance.userAuthenticateMessage('Vous avez été déconnecté. Se reconnecter ?')) {
        return this.axiosInstance(originalConfig);
      }
    } catch (_error) {
      return Promise.reject(_error);
    }
  }

  async getGiteaPublicProperties(): Promise<GiteaPublicProperties> {
    return this.axiosInstance.get<GiteaPublicProperties>(`gitea`).then((res) => res.data);
  }

  async getUserSshKey(): Promise<SshKeyPair> {
    return this.axiosInstance.put<SshKeyPair>(`users/gitea/keypair`).then((res) => res.data);
  }

  async getCurrentUser(): Promise<User> {
    return this.axiosInstance.get<User>(`auth`).then((res) => res.data);
  }

  async getUser(userId: string): Promise<User> {
    return this.axiosInstance.get<User>(`users/${userId}`).then((res) => res.data);
  }

  async getSolution(mission: string): Promise<NodeJS.ArrayBufferView> {
    return this.axiosInstance.get<NodeJS.ArrayBufferView>(`missions/${mission}/solution`).then((res) => res.data);
  }

  async grantAccessToRepository(userId: string, missionId: string): Promise<number> {
    return this.axiosInstance
      .post<MissionUser>(`users/${userId}/missions/${missionId}/repository/collaborator`)
      .then((res) => res.status);
  }

  async pingUpdateWorkTime(): Promise<MissionUser> {
    return this.axiosInstance
      .post<MissionUser>(
        `users/${(await this.getCurrentUser()).id}/missions/${UserMission.instance.missionId}/worktime`,
        {
          action: 'PING',
        },
      )
      .then((res) => res.data);
  }

  async startUpdateWorkTime() {
    return this.axiosInstance
      .post(`users/${(await this.getCurrentUser()).id}/missions/${UserMission.instance.missionId}/worktime`, {
        action: 'START',
      })
      .then((res) => res.data);
  }

  async submitAttempt(attempt: Attempt): Promise<MissionUser> {
    return this.axiosInstance
      .post<MissionUser>(
        `/users/${(await this.getCurrentUser()).id}/missions/${UserMission.instance.missionId}/solve/desktop`,
        attempt,
      )
      .then((res) => res.data);
  }
}
