import BriefingView from '../view/briefingView';
import { CommandHandler } from './commandHandler';
import ExtensionStore from './extensionStore';
import KeycloakOAuth2DeviceFlowConnection from './keycloakOAuth2DeviceFlowConnection';
import { MissionDevContainer } from './mission/missionDevContainer';
import { Command, commands, ExtensionContext, Uri, window } from 'vscode';
import { createSshKeyFiles } from './sshKeyManager';
import { emptyDirectories, removeFilesOrDirectories } from '../recorder/utils/workdir';
import { missionWorkdir, getSshFolderPath } from './config';
import { hasStatusNumber } from './utils/typeguards';
import { existsSync, mkdirSync, readdirSync, renameSync, rmSync, writeFileSync } from 'fs';
import { extract } from 'tar';
import { parse } from 'yaml';
import { clearDevContainers } from './utils/docker.util';
import isMissionStarted, { getReviewedStudentWorkdirPath } from './utils/mission.util';
import { join } from 'path';
import Mission from '../model/mission';
import ApiService from './api.service';
import StartedMissionsView from '../view/startedMissionsView';
import AuthenticationView from '../view/authenticationView';
import { error, log, warn } from './utils/log';

export default class Controller {
  private static _instance: Controller;
  private briefingView?: BriefingView;
  public readonly startedMissions?: StartedMissionsView;

  public constructor(public readonly context: ExtensionContext) {
    if (Controller._instance) {
      throw new Error('Controller is a singleton');
    }
    Controller._instance = this;
    if (isMissionStarted()) {
      this.briefingView = new BriefingView();
    } else {
      this.startedMissions = new StartedMissionsView();
    }

    this.init();
  }

  public static get instance(): Controller {
    return Controller._instance;
  }

  getChallenge(missionId: string): Mission | undefined {
    if (!isMissionStarted()) {
      return parse(`${missionWorkdir}/${missionId}/challenge.yaml`);
    }
  }

  public async getSolution(missionId: string, reviewedId: string) {
    try {
      const solution = await ApiService.instance.getSolution(missionId);
      const solutionFolder = join(getReviewedStudentWorkdirPath(reviewedId), missionId, '.solution');
      const tarName = 'solution.tar.gz';
      const tarPath = join(solutionFolder, tarName);

      if (existsSync(solutionFolder)) {
        rmSync(solutionFolder, { recursive: true });
      }

      mkdirSync(solutionFolder, { recursive: true });

      writeFileSync(tarPath, solution);
      extract({ sync: true, file: `${solutionFolder}/${tarName}`, cwd: solutionFolder });
      await removeFilesOrDirectories(`${solutionFolder}/${tarName}`);
      // list files in the folder
      readdirSync(solutionFolder).forEach((file) => {
        const tmpSolutionPath = join(solutionFolder, file);
        readdirSync(tmpSolutionPath).map((f) => renameSync(join(tmpSolutionPath, f), join(solutionFolder, f)));
        rmSync(tmpSolutionPath, { recursive: true });
      });
    } catch (e) {
      window.showErrorMessage(`Une erreur est survenue lors de la récupération de la solution`);
      error(e);
    }
  }

  private async init() {
    const that = this;
    window.registerUriHandler({
      handleUri(uri: Uri) {
        const queryParams = new URLSearchParams(uri.query);
        const action = queryParams.get('action');
        // TODO: Should we follow eslint no-case-declarations ?
        const missionId = queryParams.get('missionId');
        const missionVersion = queryParams.get('missionVersion');
        const userId = queryParams.get('userId') ?? undefined;
        const storyId = queryParams.get('storyId') ?? undefined;
        log('Opening link', uri);

        switch (action) {
          case 'open-mission':
            if (!missionId || !missionVersion) {
              window.showErrorMessage('Identifiant ou version de la mission incorrect');
            } else {
              that.launchMission({ missionId, missionVersion, revieweeId: userId, storyId });
            }

            break;

          default:
            window.showInformationMessage('Aucune action trouvée!');
        }
      },
    });
  }

  public async disconnect() {
    try {
      await removeFilesOrDirectories(`${getSshFolderPath()}/id_rsa`, `${getSshFolderPath()}/id_rsa.pub`);
    } catch (e) {
      warn('Could not remove ssh key files', e);
    }
    try {
      await emptyDirectories(missionWorkdir);
    } catch (e) {
      warn('Could not empty mission workdir', e);
    }
    try {
      await ExtensionStore.instance.clear();
    } catch (e) {
      warn('Could not clear extension store', e);
    }
    try {
      await clearDevContainers();
    } catch (e) {
      warn('Could not clear dev containers', e);
    }
    AuthenticationView.instance.isAlreadyConnected = false;
  }

  public async createSshKeyPair() {
    const { publicKey, privateKey } = await ApiService.instance.getUserSshKey();
    await createSshKeyFiles(publicKey, privateKey);
  }

  public async authenticate() {
    await KeycloakOAuth2DeviceFlowConnection.instance.registerDevice();
    const tokens = await KeycloakOAuth2DeviceFlowConnection.instance.getToken({
      openLink: Controller.openBrowserWithUrl,
    });
    await ExtensionStore.instance.setAccessToken(tokens.accessToken);
    await ExtensionStore.instance.setRefreshToken(tokens.refreshToken);
    await this.createSshKeyPair();
    AuthenticationView.instance.isAlreadyConnected = true;
  }

  public async userAuthenticateMessage(message: string): Promise<boolean> {
    const answer = await window.showInformationMessage(message, { modal: true }, 'Connexion');
    if (answer === 'Connexion') {
      await this.authenticate();
      return true;
    } else {
      return false;
    }
  }

  public static openBrowserWithUrl(url: string) {
    commands.executeCommand(CommandHandler.instance.openUrlInBrowserCommand.command, Uri.parse(url));
  }

  public async launchMission({
    missionId,
    missionVersion,
    revieweeId,
    storyId,
  }: {
    missionId: string;
    missionVersion: string;
    revieweeId?: string;
    storyId?: string;
  }) {
    window.showInformationMessage(`vous lancez la mission ${missionId}`);

    const hadBeenConnected = (await ExtensionStore.instance.getAccessToken()) !== undefined;

    if (!hadBeenConnected) {
      if (await this.userAuthenticateMessage('Connectez-vous avant de commencer la mission.')) {
        window.showInformationMessage('Connexion validée');
      } else return;
    }

    const reviewer = await ApiService.instance.getCurrentUser();

    if (!!revieweeId && revieweeId !== reviewer.id) {
      try {
        await ApiService.instance.grantAccessToRepository(revieweeId, missionId);
      } catch (e) {
        if (hasStatusNumber(e)) {
          if (e.status === 403) {
            window.showErrorMessage(`Vous n'avez pas accès à la mission: ${missionId}`);
            return;
          } else {
            window.showErrorMessage(`L'élève n'a pas commencé l'exercice: ${missionId}`);
            throw e;
          }
        } else {
          throw e;
        }
      }
      try {
        await this.getSolution(missionId, revieweeId);
      } catch (e) {
        window.showErrorMessage(`Une erreur est survenue lors de la récupération de la solution`);
        error(e);
      }
    }

    const missionDevcontainer = new MissionDevContainer(missionId, missionVersion, revieweeId, storyId);

    try {
      await missionDevcontainer.open();
    } catch (e) {
      window.showErrorMessage('Impossible de lancer la mission');
      error(e);
    }
  }
}

export const openBriefingCommand: Command = {
  title: 'Open Briefing',
  command: 'deadlock.openBriefing',
};

export const openAuthenticationCommand: Command = {
  title: 'Open Deadlock Authentication page',
  command: 'deadlock.openAuthentication',
};
