import { exec as execCallback } from 'child_process';
import simpleGit, { SimpleGit, SimpleGitOptions } from 'simple-git';
import { GITEA_PATH_IC, getSshFolderPath } from './config';
import { promisify } from 'util';
import ApiService from './api.service';
import UserMission from './mission/model/userMission';
import { error, warn } from './utils/log';
import { existsSync } from 'fs';
import { createDirectories } from '../recorder/utils/workdir';
import { homedir } from 'os';
import { join } from 'path';

const exec = promisify(execCallback);

const defaultRemote = 'origin';

type Branch = 'master' | 'live';

export default class GitMission {
  private static _instance: GitMission;
  public static get instance(): GitMission {
    if (!GitMission._instance) {
      GitMission._instance = new GitMission();
    }
    return GitMission._instance;
  }

  private git: SimpleGit;

  private constructor() {
    const options: Partial<SimpleGitOptions> = {
      baseDir: GITEA_PATH_IC,
      binary: 'git',
      maxConcurrentProcesses: 1,
    };

    this.git = simpleGit(options);
  }

  async setupSshAgent() {
    const sshFolder = join(homedir(), '.ssh');
    try {
      if (!existsSync(sshFolder)) {
        await createDirectories(sshFolder);
      }
      const gitea = await ApiService.instance.getGiteaPublicProperties();
      await exec(
        `eval "$(ssh-agent -s)" && ssh-keyscan -p ${gitea.sshPort} -H ${gitea.sshHost} >> ${join(
          sshFolder,
          'known_hosts',
        )}`,
      );
      await exec(`ssh-add ${getSshFolderPath()}/id_rsa`);
    } catch (err) {
      error('Could not add ssh keys', err);
      throw err;
    }
  }

  async forgetSshKeys() {
    try {
      await exec(`ssh-add -d ${getSshFolderPath()}/id_rsa`);
    } catch (err) {
      warn('Could not remove ssh keys', err);
    }
  }

  async getAuthor(): Promise<string> {
    return (await UserMission.instance.getGiteaUser()).username;
  }

  async init() {
    try {
      await this.setupSshAgent();

      await this.git.init();

      const remote = await this.readRemote();

      if (remote === defaultRemote) {
        return Promise.resolve(this);
      }

      const giteaUser = await UserMission.instance.getGiteaUser();

      await this.git.addRemote(defaultRemote, await this.getRemotePath());
      await this.git.addConfig('user.email', await UserMission.instance.getEmail(), false, 'local');
      await this.git.addConfig(
        'user.name',
        `${giteaUser.details.lastName} ${giteaUser.details.firstName}`,
        false,
        'local',
      );
      await this.git.addConfig('core.excludesFile', `user-git-${giteaUser.username}/\n`, true, 'local');

      return Promise.resolve(this);
    } catch (e) {
      error('Could not init git', e);
      throw e;
    }
  }

  private async getRemotePath() {
    const giteaConfig = await ApiService.instance.getGiteaPublicProperties();
    return `ssh://git@${giteaConfig.sshHost}:${giteaConfig.sshPort}/${
      (await UserMission.instance.getGiteaUser()).username
    }/${UserMission.instance.missionId}`;
  }

  async readRemote() {
    try {
      return ((await this.git.remote([])) || '').replace(/(\r\n|\n|\r)/gm, '');
    } catch (e) {
      error('Could not read remote', e);
      throw e;
    }
    return '';
  }

  fetch() {
    return this.git.fetch(defaultRemote);
  }

  pull(branch: Branch = 'master') {
    return this.git.pull(defaultRemote, branch, { '--rebase': 'true' });
  }

  addAll() {
    return this.git.add('.');
  }

  commit(message: string, options: string[] = []) {
    return this.git.commit(message, options);
  }

  push() {
    return this.git.push(defaultRemote);
  }

  createLocalBranch(branch: Branch = 'master') {
    return this.git.checkout(['-b', branch]);
  }

  async createRemoteBranch(branch: Branch = 'master') {
    return await this.git.push(['-u', defaultRemote, branch]);
  }

  setUpstream(branch: Branch = 'master') {
    return this.git.branch(['-u', defaultRemote, branch]);
  }

  createBranch(branch: Branch = 'master') {
    const createBranchLocally = this.createLocalBranch(branch);
    const createRemoteBranch = this.createRemoteBranch(branch);

    return Promise.all([createBranchLocally, createRemoteBranch]);
  }

  checkout(branch: Branch = 'master') {
    return this.git.checkout(branch);
  }

  merge(options?: string[]) {
    return this.git.merge(options ?? []);
  }

  async isRemoteRepoExist() {
    try {
      const remotes = await this.git.listRemote();
      return remotes.length !== 0;
    } catch {
      // error, Gitea throws Gitea: Unauthorized when not found
      return false;
    }
  }
}
