import { readFileSync } from 'fs';
import { writeFile } from 'fs/promises';
import { join } from 'path';
import { createDirectories } from '../../../recorder/utils/workdir';
import ApiService from '../../api.service';
import { missionWorkdir, USER_MISSION_PATH } from '../../config';
import { getReviewedStudentWorkdirPath } from '../../utils/mission.util';

interface UserMissionModel {
  missionId: string;
  missionVersion: string;
  storyId?: string;
  reviewee?: User;
}

export default class UserMission implements UserMissionModel {
  private static _instance: UserMission;

  private constructor(userMission: UserMissionModel) {
    this.missionId = userMission.missionId;
    this.missionVersion = userMission.missionVersion;
    this.reviewee = userMission.reviewee;
    this.storyId = userMission.storyId;
    UserMission._instance = this;
  }

  public static get instance(): UserMission {
    if (!UserMission._instance) {
      return (this._instance = new UserMission(JSON.parse(readFileSync(USER_MISSION_PATH, 'utf8'))));
    }
    return UserMission._instance;
  }

  public async getGiteaUser(): Promise<User & { username: string }> {
    const giteaUser: User & { username?: string } = this.reviewee ?? (await ApiService.instance.getCurrentUser());
    return { ...giteaUser, username: giteaUser.id.split('-').join('') };
  }

  public async getEmail(): Promise<string> {
    return `${(await this.getGiteaUser()).username}@deadlock.io`;
  }

  public readonly missionId: string;
  public readonly storyId?: string;
  public readonly missionVersion: string;
  public readonly reviewee?: User;

  public static getMissionUserFolder(missionId: string, revieweeId?: string): string {
    return join(revieweeId ? getReviewedStudentWorkdirPath(revieweeId) : missionWorkdir, missionId, '.config');
  }

  public static async writeFile(
    missionId: string,
    missionVersion: string,
    revieweeId?: string,
    storyId?: string,
  ): Promise<void> {
    const workdir = this.getMissionUserFolder(missionId, revieweeId);
    await createDirectories(workdir);
    const path = join(workdir, 'user-challenge.json');
    await writeFile(
      path,
      JSON.stringify(
        {
          missionId,
          storyId,
          missionVersion,
          reviewee: revieweeId ? await ApiService.instance.getUser(revieweeId) : undefined,
        } as UserMissionModel,
        null,
        2,
      ),
    );
  }

  public isReviewing(): boolean {
    return !!this.reviewee;
  }
}

export interface UserDetails {
  id: string;
  firstName: string;
  lastName: string;
  organization: string;
  email: string;
  login: string;
  roles: Set<
    'ADMIN' | 'MANAGER' | 'OFFLINE_ACCESS' | 'PROFESSOR' | 'UMA_AUTHORIZATION' | 'CANDIDATE' | 'DEFAULT-ROLES-DEADLOCK'
  >;
  avatarUrl: string;
}

export interface User {
  id: string;
  details: UserDetails;
}
