import { exec as execCallback, execSync } from 'child_process';
import { copyFileSync, existsSync, PathLike, readdirSync, renameSync } from 'fs';
import { join } from 'path';
import { promisify } from 'util';
import { mkdir, rm } from 'fs/promises';
import Controller from '../../core/controller';
import { log } from '../../core/utils/log';

const exec = promisify(execCallback);
const PREFIX = '[DEADLOCK-RECORDER]';

export async function copyProjectSources(src: string, target: string, ...excludes: Array<string>) {
  const excludeCmd = excludes.map((current) => `--exclude ${current}`).join(' ');
  return await exec(`sudo rsync -a ${src}/ ${target} ${excludeCmd} && sudo chown -R deadlock:deadlock /home/deadlock`);
}

export async function clearFilesExceptGit(path: string) {
  for (const file of readdirSync(path)) {
    if (file !== '.git') {
      const curPath = join(path, file);
      await rm(curPath, { recursive: true });
    }
  }
}

function renameIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    renameSync(srcPath, destPath);
  } else {
    log(PREFIX, `Renaming : No ${srcPath} found`);
  }
}

function copyFileIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    copyFileSync(srcPath, destPath);
  } else {
    log(PREFIX, `Copying file: No ${srcPath} found`);
  }
}

function copyFolderIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    execSync(`rsync -r ${srcPath}/* ${destPath}`);
  } else {
    log(PREFIX, `Copying folder: No ${srcPath} found`);
  }
}

/**
 *
 * Rename temporary git files back to actual git files.
 *
 * @param path Path of the folder containing user's temporary git project files.
 * @param username Identifier used to create the temporary files name
 */
export function renameTempToUserGitFiles(path: string, username: string) {
  renameIfExistsSync(join(path, `user-git-${username}`), join(path, '.git'));
  renameIfExistsSync(join(path, `user-gitignore-${username}`), join(path, '.gitignore'));
}

/**
 *
 * Copy user's git project files and paste them with renamed filenames so that
 * they can be saved by the recorder.
 *
 * @param srcPath Path of the folder containing user's git project files to be saved.
 * @param destPath Path where the user's git project files should be pasted.
 * @param username Identifier used to create the temporary files name
 */
export function copyGitUserFiles(srcPath: string, destPath: string, username: string) {
  copyFolderIfExistsSync(join(srcPath, '.git'), join(destPath, `user-git-${username}`));
  copyFileIfExistsSync(join(srcPath, '.gitignore'), join(destPath, `user-gitignore-${username}`));
}

export async function removeFilesOrDirectories(...paths: string[]) {
  for (const path of paths.filter(existsSync)) {
    await rm(path, { recursive: true });
  }
}

export async function emptyDirectories(...directoryPaths: string[]) {
  for (const directoryPath of directoryPaths) {
    await rm(directoryPath, { recursive: true });
    await mkdir(directoryPath);
  }
}

export async function createDirectories(...directoryPaths: string[]) {
  return Promise.all(directoryPaths.map((directoryPath) => mkdir(directoryPath, { recursive: true })));
}

export function getExtensionUri() {
  return Controller.instance.context.extensionUri;
}
