import { MissionCommand, MissionHandler } from '../model/mission';
import { commands, Event, EventEmitter, ThemeColor, ThemeIcon, TreeDataProvider, TreeItem, window } from 'vscode';
import { MISSION_PATH_IC } from '../core/config';
import { ChildProcessWithoutNullStreams, spawn } from 'child_process';
import AttemptBuilder from '../model/attempt';
import ApiService from '../core/api.service';
import UserMission from '../core/mission/model/userMission';
import { pushOnCommitQueueIfNotReviewing } from '../recorder/utils/gitea';
import GitMission from '../core/gitMission';

export class CommandTreeProvider implements TreeDataProvider<CommandItem> {
  getTreeItem(element: CommandItem): TreeItem {
    return element;
  }

  getChildren(element?: CommandItem): Thenable<CommandItem[]> {
    if (element) {
      return Promise.resolve([]);
    } else {
      return Promise.resolve(
        MissionHandler.instance.desktop?.scripts?.map((script) => new CommandItem(script, this)) ?? [],
      );
    }
  }

  private _onDidChangeTreeData: EventEmitter<CommandItem> = new EventEmitter<CommandItem>();
  readonly onDidChangeTreeData: Event<CommandItem> = this._onDidChangeTreeData.event;

  refresh(commandItem: CommandItem): void {
    this._onDidChangeTreeData.fire(commandItem);
  }
}

const outputPrefix = `Deadlock`;

class CommandItem extends TreeItem {
  public readonly parced: {
    cmd: string;
    args: string[];
  };

  private _isRunning: boolean;
  private spawnHandler?: ChildProcessWithoutNullStreams;

  public get isRunning(): boolean {
    return this._isRunning;
  }

  private set isRunning(value: boolean) {
    this._isRunning = value;
    if (this._isRunning) {
      this.apply();
    } else {
      if (this.spawnHandler?.exitCode === null) {
        this.spawnHandler?.kill();
      }
      this.spawnHandler = undefined;
    }
    this.iconPath = new ThemeIcon(
      this._isRunning ? 'notebook-stop' : 'notebook-execute',
      new ThemeColor(this.missionCommand.type === 'submit' ? 'debugIcon.stopForeground' : 'debugIcon.startForeground'),
    );
    this.parent.refresh(this);
  }

  constructor(private readonly missionCommand: MissionCommand, private readonly parent: CommandTreeProvider) {
    super(missionCommand.name);
    missionCommand.type = missionCommand.type ?? 'run';
    this.tooltip = missionCommand.description ?? missionCommand.name;
    this.description = missionCommand.description;
    this._isRunning = false;
    this.iconPath = new ThemeIcon(
      'notebook-execute',
      new ThemeColor(missionCommand.type === 'submit' ? 'debugIcon.stopForeground' : 'debugIcon.startForeground'),
    );
    this.command = {
      title: missionCommand.name,
      command: missionCommand.name,
      tooltip: this.tooltip,
    };
    const split = missionCommand.command.split(' ').map((s) => s.replace(/\$([^$/]+)/, (_, n) => process.env[n] ?? ''));
    this.parced = { cmd: split[0], args: split.slice(1) };
    commands.registerCommand(missionCommand.name, () => {
      if (this.isRunning) {
        this.isRunning = false;
      } else {
        if (missionCommand.type === 'submit') {
          window
            .showInformationMessage('Êtes-vous sûr de soumettre votre solution ?', { modal: true }, 'Oui')
            .then((answer) => {
              if (answer === 'Oui') {
                this.isRunning = true;
              }
            });
        } else {
          this.isRunning = true;
        }
      }
    });
  }

  async apply() {
    const output = window.createOutputChannel(`${outputPrefix} - ${this.missionCommand.name}`);
    output.show();
    const attempt = new AttemptBuilder(
      (await ApiService.instance.getCurrentUser()).id,
      UserMission.instance.missionId,
      this.missionCommand.type === 'submit',
      UserMission.instance.storyId,
    );
    this.spawnHandler = spawn(this.parced.cmd, this.parced.args, {
      cwd: MISSION_PATH_IC,
    });
    this.spawnHandler.stdout.addListener('data', (data) => {
      output.append(data.toString());
      attempt.addLog(data.toString());
    });
    this.spawnHandler.stderr.addListener('data', (data) => {
      output.append(data.toString());
      attempt.addLog(data.toString());
    });
    this.spawnHandler.addListener('exit', async (code) => {
      output.appendLine(`${outputPrefix} - ${this.missionCommand.name} - exited with code ${code}`);
      attempt.setExitCode(code ?? undefined);
      if (this.missionCommand.type === 'submit' || this.missionCommand.type === 'run') {
        pushOnCommitQueueIfNotReviewing(GitMission.instance, 'Run', async (commit) => {
          await ApiService.instance.submitAttempt(attempt.build(commit?.commit));
        });
      }
      this.isRunning = false;
    });
  }
}
