import { randomBytes } from 'crypto';
import { commands, Webview, WebviewView, WebviewViewProvider } from 'vscode';
import { CommandHandler } from '../core/commandHandler';
import ExtensionStore from '../core/extensionStore';
import { getExtensionUri } from '../recorder/utils/workdir';
import { getUri } from './webViewBase';

export const authenticationId = 'authentication';

export default class AuthenticationView implements WebviewViewProvider {
  // Make as singleton
  private static _instance?: AuthenticationView;

  public static get instance(): AuthenticationView {
    if (!AuthenticationView._instance) {
      AuthenticationView._instance = new AuthenticationView();
    }
    return AuthenticationView._instance;
  }

  private webviewView?: WebviewView;

  private _isAlreadyConnected: boolean;

  private constructor() {
    this._isAlreadyConnected = false;
  }

  resolveWebviewView(webviewView: WebviewView): void | Thenable<void> {
    this.webviewView = webviewView;
    webviewView.webview.options = {
      enableScripts: true,
      localResourceRoots: [getExtensionUri()],
    };
    webviewView.webview.html = this.render(webviewView.webview);
    webviewView.webview.onDidReceiveMessage(this.onMessageReceive, this);
    ExtensionStore.instance.getAccessToken().then((token?: string) => {
      this.isAlreadyConnected = token !== undefined;
    });
  }

  set isAlreadyConnected(newVal: boolean) {
    if (this._isAlreadyConnected !== newVal) {
      this._isAlreadyConnected = newVal;
      this.reload();
    }
  }

  reload(): void {
    if (this.webviewView) {
      this.webviewView.webview.html = this.render(this.webviewView.webview);
    }
  }

  onMessageReceive(message: any): void {
    switch (message.command) {
      case 'openAuthenticationPageAction':
        commands.executeCommand(CommandHandler.instance.authenticateCommand.command);
        return;
      case 'disconnectUserAction':
        commands.executeCommand(CommandHandler.instance.disconnectCommand.command);
        return;
    }
  }

  render(webview: Webview) {
    const toolkitUri = getUri(webview, ['node_modules', '@vscode', 'webview-ui-toolkit', 'dist', 'toolkit.js']);

    const css = getUri(webview, ['resources', 'styles', 'authenticationView.css']);

    const js = getUri(webview, ['resources', 'js', 'authenticationView.js']);

    const html = `<vscode-button onclick="${
      this._isAlreadyConnected ? 'disconnectUserAction' : 'openAuthenticationPageAction'
    }()">${this._isAlreadyConnected ? 'Déconnexion de' : 'Connexion à'} Deadlock
      </vscode-button>`;

    return `<!DOCTYPE html>
            <html lang="en">
            <head>
              <meta charset="UTF-8">
              <script type="module" src="${toolkitUri}"></script>
              <script nonce="${randomBytes(16).toString('base64')}" src="${js}"></script>
              <link rel="stylesheet" type="text/css" href="${css}">
            </head>
            <body>
              <div>
                ${html}
              </div>
            </body>
            </html>`;
  }
}
