import { readFileSync } from 'fs';
import { marked } from 'marked';
import { join } from 'path';
import { setInterval } from 'timers';
import { Uri, workspace } from 'vscode';
import ApiService from '../core/api.service';
import { BRIEFING_FILE_NAME, DOCS_PATH_IC } from '../core/config';
import { openBriefingCommand } from '../core/controller';
import UserMission from '../core/mission/model/userMission';
import isMissionStarted from '../core/utils/mission.util';
import { MissionHandler } from '../model/mission';
import { getUri, WebViewBase } from './webViewBase';

export const briefingId = 'brefingView';

export default class BriefingView extends WebViewBase {
  private briefingContent: string | undefined;

  private errorLoadingBriefing = false;

  private loadingBriefing = false;

  constructor() {
    super(briefingId, 'Briefing', openBriefingCommand);
  }

  toBase64(file: string) {
    const result = readFileSync(join(DOCS_PATH_IC, file), {
      encoding: 'base64',
    });
    return result;
  }

  setupImages(text: string) {
    //eslint-disable-next-line
    const imageRegex = /\!\[(.*?)\]\(\b(image:)(.*?\))?/g;

    return text.replace(imageRegex, (match, title, imagePrefix, img) => {
      const fileName = img.replace(')', '');
      const extension = fileName.split('.');

      return `![${title}](data:image/${extension[extension.length - 1]};base64,${this.toBase64(fileName)})`;
    });
  }

  load(): void {
    const intervalId = setInterval(() => {
      if (workspace && !this.loadingBriefing) {
        this.loadingBriefing = true;
        clearInterval(intervalId);

        workspace.openTextDocument(Uri.parse(join(DOCS_PATH_IC, BRIEFING_FILE_NAME))).then(
          (document) => {
            try {
              this.briefingContent = this.setupImages(document.getText());
            } catch (e) {
              console.error(e);
              this.briefingContent = 'Error while parsing your briefing.';
            }
            if (isMissionStarted()) this.show();
          },
          (error) => {
            console.error('Cannot load briefing', error);
            this.errorLoadingBriefing = true;
          },
        );
      }
    }, 500);
  }

  async render() {
    this.load();
    let output = '';

    const missionUser = UserMission.instance;

    if (missionUser.isReviewing()) {
      const user = await ApiService.instance.getCurrentUser();
      const giteaUser = await missionUser.getGiteaUser();
      output += `
        <h2>Professeur</h2>
        Bonjour <b>${user.details.lastName} ${user.details.firstName}</b> vous êtes actuellement entrain de relire le code de <b>${giteaUser.details.lastName} ${giteaUser.details.firstName}</b>.<br />
        Vous pouvez consulter chaque exécution de son code dans l'onglet à gauche de GitLens. Chaque commit sur <b>master</b> représente une exécution.
        <br />
      `;
    } else {
      if (MissionHandler.instance.desktop?.scripts) {
        output += `
          <h1>Comment ça marche Deadlock ?</h2>
          <h3>Soumettre son code</h3>
          Cet exercice possède des commandes de test et/ou de validation. Vous pouvez y accéder depuis l'extension dans l'onglet COMMANDS.
          <ul>
          <li>Les commandes de <b>test</b> (avec triangle vert) permettent de tester son code avant une soumission au professeur et permettent de déboguer.<br/> </li>
          <li>Les commandes de <b>submit</b> (avec triangle rouge) permettent de valider son code et d'indiquer au professeur/reviewer/employeur que l'exercice est terminé et prêt à être relu/évalué.
          <br />
          <b>Attention !</b> Lors d'un examen, ce bouton ne peut être utilisé qu'une seule fois. L'exercice sera alors considéré comme terminé définitivement.</li>
          </ul>
          <img src="${getUri(this.panel!.webview, [
            'media',
            'commands.gif',
          ])}" alt="Les commandes sont accessibles dans l'onglet COMMANDS de l'extension Deadlock">
          <br />
        `;
      }
    }

    if (this.briefingContent) {
      output += `<h1>Mission Goal 🕶</h1>${marked(this.briefingContent)}`;
    } else if (this.errorLoadingBriefing) {
      output += 'Cannot load briefing.';
    } else {
      output += 'Loading briefing..';
    }

    output += '<br />';

    return output;
  }
}
