import { randomBytes } from 'crypto';
import { existsSync, readdirSync, readFileSync } from 'fs';
import { join } from 'path';
import { Webview, WebviewView, WebviewViewProvider } from 'vscode';
import Controller from '../core/controller';
import { getUri } from './webViewBase';
import { missionWorkdir } from '../core/config';
import { watch } from 'chokidar';
import UserMission from '../core/mission/model/userMission';
import { getExtensionUri } from '../recorder/utils/workdir';
import { warn } from '../core/utils/log';

export default class StartedMissionsView implements WebviewViewProvider {
  constructor() {
    if (existsSync(missionWorkdir)) {
      console.log('Watching mission directory');
      const watcher = watch(missionWorkdir, {
        ignoreInitial: true,
        ignorePermissionErrors: true,
        depth: 3,
      });
      watcher.on('all', (event, path) => {
        if (event === 'add' || event === 'unlink' || (event === 'change' && path.includes('user-challenge.json'))) {
          if (this.webviewView?.webview.html) {
            this.webviewView.webview.html = this._getHtmlForWebview(this.webviewView?.webview);
          }
        }
      });
    }
  }
  private webviewView?: WebviewView;
  public refresh(): void {
    if (this.webviewView) {
      this.webviewView.webview.html = this._getHtmlForWebview(this.webviewView?.webview);
    }
  }
  resolveWebviewView(webviewView: WebviewView): void | Thenable<void> {
    this.webviewView = webviewView;
    webviewView.webview.options = {
      enableScripts: true,
      localResourceRoots: [getExtensionUri()],
    };
    webviewView.webview.html = this._getHtmlForWebview(webviewView.webview);
    webviewView.webview.onDidReceiveMessage(this.onMessageReceive.bind(this));
  }

  onMessageReceive(message: any): void {
    switch (message.command) {
      case 'openMission': {
        const path = join(missionWorkdir, message.mission, '.config', 'user-challenge.json');
        const userMission: UserMission = JSON.parse(readFileSync(path, 'utf8'));
        Controller.instance.launchMission({
          missionId: userMission.missionId,
          missionVersion: userMission.missionVersion,
          storyId: userMission.storyId,
        });
        return;
      }
    }
  }

  private _getHtmlForWebview(webview: Webview) {
    const toolkitUri = getUri(webview, ['node_modules', '@vscode', 'webview-ui-toolkit', 'dist', 'toolkit.js']);

    const css = getUri(webview, ['resources', 'styles', 'startedMissionsView.css']);

    const js = getUri(webview, ['resources', 'js', 'startedMissionsView.js']);

    // find all folders in mission directory
    let missionsHtml = '';
    existsSync(missionWorkdir) &&
      readdirSync(missionWorkdir).forEach((mission) => {
        if (existsSync(join(missionWorkdir, mission, '.config', 'user-challenge.json'))) {
          try {
            const userChallenge: UserMission = JSON.parse(
              readFileSync(join(missionWorkdir, mission, '.config', 'user-challenge.json'), 'utf8'),
            );
            missionsHtml += `<vscode-button onclick="openMission('${userChallenge.missionId}')" class="item" appearance="primary">${userChallenge.missionId}</vscode-button>`;
          } catch (e) {
            warn(`Failed to read user-challenge.json for mission ${mission}`);
          }
        } else {
          warn(`Folder '${mission}' has no user-challenge.json file so it is ignored.`);
        }
      });

    if (!missionsHtml) {
      missionsHtml = '<div class="item">Aucune mission n\'a été démarrée</div>';
    }

    return `<!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <script type="module" src="${toolkitUri}"></script>
        <script nonce="${randomBytes(16).toString('base64')}" src="${js}"></script>
        <link rel="stylesheet" type="text/css" href="${css}">
        </head>
    <body>
      <div>
        ${missionsHtml}
      </div>
    </body>
    </html>`;
  }
}
